from contextlib import asynccontextmanager
from datetime import datetime, timedelta
from typing import AsyncGenerator

from loguru import logger
from aiohttp import ClientSession
from pydantic import BaseModel

from .common import APIClient
from ..api import (
	CalculatorMixin,
	OrdersMixin,
	LocationsMixin,
	DeliveryPointsMixin,
	CourierMixin,
	PrintingMixin,
	WebhooksMixin,
)


class CDEKAuthData(BaseModel):
	access_token: str
	expires_at: datetime

	def is_expired(self):
		return datetime.now() > self.expires_at


class CDEKAPIClient(
	APIClient,
	CalculatorMixin,
	OrdersMixin,
	LocationsMixin,
	DeliveryPointsMixin,
	CourierMixin,
	PrintingMixin,
	WebhooksMixin,
):
	_cached_auth_data: CDEKAuthData = None
	_client_name = __qualname__
	_client_id: str = None
	_client_secret: str = None

	def __init__(self, client_id: str, client_secret: str, test_environment: bool = False, debug: bool = False):
		self._client_id = client_id
		self._client_secret = client_secret
		self._base_url = "https://api.cdek.ru/" if not test_environment else "https://api.edu.cdek.ru/"
		self._debug_mode = debug

	@asynccontextmanager
	async def _get_cached_auth_data(self) -> AsyncGenerator[CDEKAuthData | None]:
		if self._cached_auth_data and not self._cached_auth_data.is_expired():
			logger.debug("Retrieved cached AuthData")
			yield self._cached_auth_data
			return

		logger.debug("Requesting new AuthData")
		async with ClientSession() as session:
			auth_request = await session.post(
				"https://api.cdek.ru/v2/oauth/token",
				data={
					"grant_type": "client_credentials",
					"client_id": self._client_id,
					"client_secret": self._client_secret,
				},
			)
			if auth_request.ok:
				data = await auth_request.json()
				access_token = data["access_token"]
				expires_at = datetime.now() + timedelta(seconds=data["expires_in"])
				self._cached_auth_data = CDEKAuthData(
					access_token=access_token, expires_at=expires_at
				)
				yield self._cached_auth_data
			else:
				logger.error(await auth_request.text())
				yield None



from .orders import OrdersMixin
from .courier import CourierMixin
from .printing import PrintingMixin
from .webhooks import WebhooksMixin
from .locations import LocationsMixin
from .calculator import CalculatorMixin
from .delivery_points import DeliveryPointsMixin


__all__ = [
	"OrdersMixin",
	"CourierMixin",
	"PrintingMixin",
	"WebhooksMixin",
	"LocationsMixin",
	"CalculatorMixin",
	"DeliveryPointsMixin",
]



from pydantic import BaseModel, Field
from ..enums import PrintFormat


class PrintRequest(BaseModel):
	orders: list[str] = Field(..., description="List of order UUIDs")
	format: PrintFormat = Field(PrintFormat.A4, description="Print format")

	def to_query_params(self) -> dict:
		return {"orders": ",".join(self.orders), "format": self.format}


class PrintResponse(BaseModel):
	uuid: str = Field(..., description="Print job UUID")
	url: str | None = Field(None, description="Download URL for print file")
	statuses: list[dict] | None = Field(None, description="Print job statuses")
	errors: list[dict] | None = Field(None, description="Print job errors")



from pydantic import ValidationError
from loguru import logger
from ..models import (
	DeliveryPointFilter,
	DeliveryPoint,
	DeliveryPointSearchResult,
)


class DeliveryPointsMixin:
	async def get_delivery_points(
		self, filter_params: DeliveryPointFilter | None = None
	) -> list[DeliveryPoint]:
		try:
			params = (
				filter_params.model_dump(exclude_none=True) if filter_params else {}
			)
			response = await self._get("/v2/deliverypoints", params=params)
			if isinstance(response, list):
				return [DeliveryPoint(**point) for point in response]
			return [DeliveryPoint(**response)]
		except ValidationError as e:
			logger.error(f"Validation error in get_delivery_points: {e}")
			raise

	async def search_delivery_points(
		self, filter_params: DeliveryPointFilter | None = None
	) -> DeliveryPointSearchResult:
		try:
			params = (
				filter_params.model_dump(exclude_none=True) if filter_params else {}
			)
			response = await self._get("/v2/deliverypoints", params=params)
			return DeliveryPointSearchResult(**response)
		except ValidationError as e:
			logger.error(f"Validation error in search_delivery_points: {e}")
			raise



from enum import IntEnum, StrEnum


class OrderType(IntEnum):
	ONLINE_STORE = 1
	REGULAR_DELIVERY = 2

class TariffCode(IntEnum):
	EXPRESS_DOOR_TO_DOOR = 1
	EXPRESS_DOOR_TO_PICKUP = 2
	EXPRESS_PICKUP_TO_DOOR = 3
	EXPRESS_PICKUP_TO_PICKUP = 4
	ECONOMY_DOOR_TO_DOOR = 5
	ECONOMY_DOOR_TO_PICKUP = 6
	ECONOMY_PICKUP_TO_DOOR = 7
	ECONOMY_PICKUP_TO_PICKUP = 8
	SUPER_EXPRESS_9_18 = 10
	SUPER_EXPRESS_10_16 = 11
	SUPER_EXPRESS_18_20 = 12
	OVERNIGHT_EXPRESS = 15
	ECONOMY_EXPRESS = 16
	CARGO_DOOR_TO_DOOR = 62
	CARGO_DOOR_TO_PICKUP = 63
	CARGO_PICKUP_TO_DOOR = 64
	CARGO_PICKUP_TO_PICKUP = 65
	EXPRESS_LIGHT = 136
	POSTAMAT = 137
	ECONOMY_PICKUP_TO_POSTAMAT = 138
	SUPER_EXPRESS_9_18_DOCUMENTS = 233
	ECONOMY_EXPRESS_PICKUP_TO_PICKUP = 234
	ECONOMY_EXPRESS_DOOR_TO_PICKUP = 235
	ECONOMY_EXPRESS_PICKUP_TO_DOOR = 236
	ECONOMY_EXPRESS_DOOR_TO_DOOR = 291
	ECONOMY_PICKUP_TO_PICKUP_STANDARD = 293
	ECONOMY_DOOR_TO_PICKUP_STANDARD = 294
	ECONOMY_PICKUP_TO_DOOR_STANDARD = 295
	ECONOMY_DOOR_TO_DOOR_STANDARD = 366
	ECONOMY_EXPRESS_WAREHOUSE = 368
	ECONOMY_EXPRESS_WAREHOUSE_PICKUP = 378
	POSTAMAT_DOOR = 480
	POSTAMAT_PICKUP = 481
	EXPRESS_LIGHT_PICKUP = 482
	EXPRESS_LIGHT_DOOR = 483
	CARGO_EXPRESS = 485
	CARGO_SUPER_EXPRESS = 486


class DeliveryMode(StrEnum):
	DOOR = "door"
	PICKUP = "pickup"
	POSTAMAT = "postamat"


class PackageType(StrEnum):
	PACKAGE = "package"
	ENVELOPE = "envelope"
	CARGO = "cargo"


class ServiceCode(StrEnum):
	INSURANCE = "INSURANCE"
	DANGEROUS_GOODS = "DANGEROUS_GOODS"
	TRYING_ON = "TRYING_ON"
	PARTIAL_DELIVERY = "PARTIAL_DELIVERY"
	INSPECTION_CARGO = "INSPECTION_CARGO"
	REVERSE_SHIPMENT = "REVERSE_SHIPMENT"
	DELIVERY_TO_FLOOR = "DELIVERY_TO_FLOOR"
	SMS_NOTIFICATION = "SMS_NOTIFICATION"
	CALL_NOTIFICATION = "CALL_NOTIFICATION"
	FRAGILE_CARGO = "FRAGILE_CARGO"
	OVERSIZED_CARGO = "OVERSIZED_CARGO"
	HEAVY_CARGO = "HEAVY_CARGO"
	TEMPERATURE_MODE = "TEMPERATURE_MODE"
	PAYMENT_CARD = "PAYMENT_CARD"
	CARGO_CHECK = "CARGO_CHECK"
	RETURN_RECEIPT = "RETURN_RECEIPT"
	DOCUMENTS_COPY = "DOCUMENTS_COPY"
	PHOTO_DOCUMENT = "PHOTO_DOCUMENT"
	PERSONAL_DELIVERY = "PERSONAL_DELIVERY"
	NOTIFICATIONS = "NOTIFICATIONS"
	COURIER_PACKAGE_A2 = "COURIER_PACKAGE_A2"
	COURIER_PACKAGE_A3 = "COURIER_PACKAGE_A3"
	COURIER_PACKAGE_A4 = "COURIER_PACKAGE_A4"
	COURIER_PACKAGE_A5 = "COURIER_PACKAGE_A5"
	SECURE_PACKAGE_A3 = "SECURE_PACKAGE_A3"
	SECURE_PACKAGE_A4 = "SECURE_PACKAGE_A4"
	SECURE_PACKAGE_A5 = "SECURE_PACKAGE_A5"


class OrderStatus(StrEnum):
	CREATED = "CREATED"
	ACCEPTED = "ACCEPTED"
	READY_FOR_SHIPMENT = "READY_FOR_SHIPMENT"
	SHIPPED = "SHIPPED"
	IN_TRANSIT = "IN_TRANSIT"
	IN_CITY = "IN_CITY"
	ARRIVED_AT_DELIVERY_POINT = "ARRIVED_AT_DELIVERY_POINT"
	DELIVERED = "DELIVERED"
	NOT_DELIVERED = "NOT_DELIVERED"
	RETURNED = "RETURNED"
	PARTIALLY_DELIVERED = "PARTIALLY_DELIVERED"
	CANCELLED = "CANCELLED"
	INVALID = "INVALID"


class PackageStatus(StrEnum):
	CREATED = "CREATED"
	ACCEPTED = "ACCEPTED"
	READY_FOR_SHIPMENT = "READY_FOR_SHIPMENT"
	SHIPPED = "SHIPPED"
	IN_TRANSIT = "IN_TRANSIT"
	IN_CITY = "IN_CITY"
	ARRIVED_AT_DELIVERY_POINT = "ARRIVED_AT_DELIVERY_POINT"
	DELIVERED = "DELIVERED"
	NOT_DELIVERED = "NOT_DELIVERED"
	RETURNED = "RETURNED"
	CANCELLED = "CANCELLED"


class DeliveryPointType(StrEnum):
	PVZ = "PVZ"
	POSTAMAT = "POSTAMAT"
	ALL = "ALL"


class PaymentMethod(StrEnum):
	CASH = "CASH"
	CARD = "CARD"


class WeightLimit(IntEnum):
	MIN_WEIGHT = 1
	MAX_WEIGHT_EXPRESS = 30000
	MAX_WEIGHT_ECONOMY = 50000
	MAX_WEIGHT_CARGO = 1500000


class DimensionLimit(IntEnum):
	MIN_DIMENSION = 1
	MAX_DIMENSION_EXPRESS = 150
	MAX_DIMENSION_ECONOMY = 200
	MAX_DIMENSION_CARGO = 300


class CountryCode(StrEnum):
	AD = "AD"
	AE = "AE"
	AF = "AF"
	AG = "AG"
	AI = "AI"
	AL = "AL"
	AM = "AM"
	AO = "AO"
	AQ = "AQ"
	AR = "AR"
	AS = "AS"
	AT = "AT"
	AU = "AU"
	AW = "AW"
	AX = "AX"
	AZ = "AZ"
	BA = "BA"
	BB = "BB"
	BD = "BD"
	BE = "BE"
	BF = "BF"
	BG = "BG"
	BH = "BH"
	BI = "BI"
	BJ = "BJ"
	BL = "BL"
	BM = "BM"
	BN = "BN"
	BO = "BO"
	BQ = "BQ"
	BR = "BR"
	BS = "BS"
	BT = "BT"
	BV = "BV"
	BW = "BW"
	BY = "BY"
	BZ = "BZ"
	CA = "CA"
	CC = "CC"
	CD = "CD"
	CF = "CF"
	CG = "CG"
	CH = "CH"
	CI = "CI"
	CK = "CK"
	CL = "CL"
	CM = "CM"
	CN = "CN"
	CO = "CO"
	CR = "CR"
	CU = "CU"
	CV = "CV"
	CW = "CW"
	CX = "CX"
	CY = "CY"
	CZ = "CZ"
	DE = "DE"
	DJ = "DJ"
	DK = "DK"
	DM = "DM"
	DO = "DO"
	DZ = "DZ"
	EC = "EC"
	EE = "EE"
	EG = "EG"
	EH = "EH"
	ER = "ER"
	ES = "ES"
	ET = "ET"
	FI = "FI"
	FJ = "FJ"
	FK = "FK"
	FM = "FM"
	FO = "FO"
	FR = "FR"
	GA = "GA"
	GB = "GB"
	GD = "GD"
	GE = "GE"
	GF = "GF"
	GG = "GG"
	GH = "GH"
	GI = "GI"
	GL = "GL"
	GM = "GM"
	GN = "GN"
	GP = "GP"
	GQ = "GQ"
	GR = "GR"
	GS = "GS"
	GT = "GT"
	GU = "GU"
	GW = "GW"
	GY = "GY"
	HK = "HK"
	HM = "HM"
	HN = "HN"
	HR = "HR"
	HT = "HT"
	HU = "HU"
	ID = "ID"
	IE = "IE"
	IL = "IL"
	IM = "IM"
	IN = "IN"
	IO = "IO"
	IQ = "IQ"
	IR = "IR"
	IS = "IS"
	IT = "IT"
	JE = "JE"
	JM = "JM"
	JO = "JO"
	JP = "JP"
	KE = "KE"
	KG = "KG"
	KH = "KH"
	KI = "KI"
	KM = "KM"
	KN = "KN"
	KP = "KP"
	KR = "KR"
	KW = "KW"
	KY = "KY"
	KZ = "KZ"
	LA = "LA"
	LB = "LB"
	LC = "LC"
	LI = "LI"
	LK = "LK"
	LR = "LR"
	LS = "LS"
	LT = "LT"
	LU = "LU"
	LV = "LV"
	LY = "LY"
	MA = "MA"
	MC = "MC"
	MD = "MD"
	ME = "ME"
	MF = "MF"
	MG = "MG"
	MH = "MH"
	MK = "MK"
	ML = "ML"
	MM = "MM"
	MN = "MN"
	MO = "MO"
	MP = "MP"
	MQ = "MQ"
	MR = "MR"
	MS = "MS"
	MT = "MT"
	MU = "MU"
	MV = "MV"
	MW = "MW"
	MX = "MX"
	MY = "MY"
	MZ = "MZ"
	NA = "NA"
	NC = "NC"
	NE = "NE"
	NF = "NF"
	NG = "NG"
	NI = "NI"
	NL = "NL"
	NO = "NO"
	NP = "NP"
	NR = "NR"
	NU = "NU"
	NZ = "NZ"
	OM = "OM"
	PA = "PA"
	PE = "PE"
	PF = "PF"
	PG = "PG"
	PH = "PH"
	PK = "PK"
	PL = "PL"
	PM = "PM"
	PN = "PN"
	PR = "PR"
	PS = "PS"
	PT = "PT"
	PW = "PW"
	PY = "PY"
	QA = "QA"
	RE = "RE"
	RO = "RO"
	RS = "RS"
	RU = "RU"
	RW = "RW"
	SA = "SA"
	SB = "SB"
	SC = "SC"
	SD = "SD"
	SE = "SE"
	SG = "SG"
	SH = "SH"
	SI = "SI"
	SJ = "SJ"
	SK = "SK"
	SL = "SL"
	SM = "SM"
	SN = "SN"
	SO = "SO"
	SR = "SR"
	SS = "SS"
	ST = "ST"
	SV = "SV"
	SX = "SX"
	SY = "SY"
	SZ = "SZ"
	TC = "TC"
	TD = "TD"
	TF = "TF"
	TG = "TG"
	TH = "TH"
	TJ = "TJ"
	TK = "TK"
	TL = "TL"
	TM = "TM"
	TN = "TN"
	TO = "TO"
	TR = "TR"
	TT = "TT"
	TV = "TV"
	TW = "TW"
	TZ = "TZ"
	UA = "UA"
	UG = "UG"
	UM = "UM"
	US = "US"
	UY = "UY"
	UZ = "UZ"
	VA = "VA"
	VC = "VC"
	VE = "VE"
	VG = "VG"
	VI = "VI"
	VN = "VN"
	VU = "VU"
	WF = "WF"
	WS = "WS"
	YE = "YE"
	YT = "YT"
	ZA = "ZA"
	ZM = "ZM"
	ZW = "ZW"
	OS = "OS"
	AB = "AB"
	XK = "XK"
	AN = "AN"
	RC = "RC"
	nil = ""


class Currency(StrEnum):
	RUB = "RUB"
	KZT = "KZT"
	USD = "USD"
	EUR = "EUR"
	GBP = "GBP"
	CNY = "CNY"
	BYR = "BYR"
	UAH = "UAH"
	KGS = "KGS"
	AMD = "AMD"
	TRY = "TRY"
	THB = "THB"
	KRW = "KRW"
	AED = "AED"
	UZS = "UZS"
	MNT = "MNT"
	PLN = "PLN"
	AZN = "AZN"
	GEL = "GEL"
	JPY = "JPY"
	VND = "VND"


class VATRate(IntEnum):
	ZERO = 0
	TEN = 10
	TWENTY = 20


class PrintFormat(StrEnum):
	A4 = "A4"
	A5 = "A5"
	A6 = "A6"


class WebhookType(StrEnum):
	ORDER_STATUS = "ORDER_STATUS"
	PRINT_FORM = "PRINT_FORM"
	DELIVERY_STATUS = "DELIVERY_STATUS"


class Language(StrEnum):
	RU = "rus"
	EN = "eng"
	ZH = "zho"



from pydantic import ValidationError
from loguru import logger
from ..models import (
	OrderRequest,
	OrderResponse,
	OrderInfo,
	OrderSearchParams,
)


class OrdersMixin:
	async def create_order(self, order: OrderRequest) -> OrderResponse:
		try:
			response = await self._post(
				"/v2/orders", json=order.model_dump(exclude_none=True)
			)
			return OrderResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in create_order: {e}")
			raise

	async def get_order(self, uuid: str) -> OrderInfo:
		try:
			response = await self._get(f"/v2/orders/{uuid}")
			return OrderInfo(**response)
		except ValidationError as e:
			logger.error(f"Validation error in get_order: {e}")
			raise

	async def get_order_by_cdek_number(self, cdek_number: str) -> list[OrderInfo]:
		try:
			response = await self._get("/v2/orders", params={"cdek_number": cdek_number})
			if isinstance(response, list):
				return [OrderInfo(**order) for order in response]
			return [OrderInfo(**response)]
		except ValidationError as e:
			logger.error(f"Validation error in get_order_by_cdek_number: {e}")
			raise

	async def get_orders(
		self, params: OrderSearchParams | None = None
	) -> list[OrderInfo]:
		try:
			search_params = params.model_dump(exclude_none=True) if params else {}
			response = await self._get("/v2/orders", params=search_params)
			if isinstance(response, list):
				return [OrderInfo(**order) for order in response]
			return [OrderInfo(**response)]
		except ValidationError as e:
			logger.error(f"Validation error in get_orders: {e}")
			raise

	async def update_order(self, uuid: str, order: OrderRequest) -> OrderResponse:
		try:
			response = await self._put(
				f"/v2/orders/{uuid}", json=order.model_dump(exclude_none=True)
			)
			return OrderResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in update_order: {e}")
			raise

	async def delete_order(self, uuid: str) -> OrderResponse:
		try:
			response = await self._delete(f"/v2/orders/{uuid}")
			return OrderResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in delete_order: {e}")
			raise



from pydantic import ValidationError
from loguru import logger

from ..enums import PrintFormat
from ..models import (
	PrintRequest,
	PrintResponse,
)


class PrintingMixin:
	async def get_order_print_form(
		self, order_uuids: list[str], format: PrintFormat = PrintFormat.A4
	) -> PrintResponse:
		try:
			print_request = PrintRequest(orders=order_uuids, format=format)
			response = await self._get(
				"/v2/print/orders", params=print_request.to_query_params()
			)
			return PrintResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in get_order_print_form: {e}")
			raise

	async def get_order_barcode(
		self, order_uuids: list[str], format: PrintFormat = PrintFormat.A4
	) -> PrintResponse:
		try:
			print_request = PrintRequest(orders=order_uuids, format=format)
			response = await self._get(
				"/v2/print/barcodes", params=print_request.to_query_params()
			)
			return PrintResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in get_order_barcode: {e}")
			raise



from pydantic import BaseModel, Field
from ..enums import DeliveryPointType, CountryCode, Language


class DeliveryPointFilter(BaseModel):
	postal_code: str | None = Field(None, description="Postal code")
	city_code: int | None = Field(None, description="City code")
	type: DeliveryPointType | None = Field(
		None, description="Point type (PVZ, POSTAMAT)"
	)
	country_code: CountryCode | None = Field(None, description="Country code")
	region_code: int | None = Field(None, description="Region code")
	have_cashless: bool | None = Field(None, description="Cashless payment available")
	have_cash: bool | None = Field(None, description="Cash payment available")
	allowed_cod: bool | None = Field(None, description="Cash on delivery allowed")
	is_dressing_room: bool | None = Field(None, description="Has dressing room")
	weight_max: int | None = Field(None, description="Maximum weight limit")
	lang: Language | None = Field(Language.RU, description="Language")
	take_only: bool | None = Field(None, description="Take only (no reception)")


class WorkTime(BaseModel):
	day: int = Field(..., description="Day of week (1-7)")
	time: str = Field(..., description="Working hours (HH:MM-HH:MM)")


class DeliveryPoint(BaseModel):
	code: str = Field(..., description="Point code")
	name: str = Field(..., description="Point name")
	uuid: str | None = Field(None, description="Point UUID")
	location: dict = Field(..., description="Location information")
	address_comment: str | None = Field(None, description="Address comment")
	nearest_station: str | None = Field(None, description="Nearest metro station")
	nearest_metro_station: str | None = Field(None, description="Nearest metro station")
	work_time: str | None = Field(None, description="Working hours")
	work_time_list: list[WorkTime] | None = Field(
		None, description="Detailed working hours"
	)
	work_time_exceptions: list[dict] | None = Field(
		None, description="Working hours exceptions"
	)
	phones: list[dict] | None = Field(None, description="Phone numbers")
	email: str | None = Field(None, description="Email")
	note: str | None = Field(None, description="Additional notes")
	type: DeliveryPointType = Field(..., description="Point type")
	owner_code: str | None = Field(None, description="Owner code")
	take_only: bool = Field(..., description="Take only (no reception)")
	is_handout: bool = Field(..., description="Is handout point")
	is_reception: bool = Field(..., description="Is reception point")
	is_dressing_room: bool = Field(..., description="Has dressing room")
	have_cashless: bool = Field(..., description="Cashless payment available")
	have_cash: bool = Field(..., description="Cash payment available")
	allowed_cod: bool = Field(..., description="Cash on delivery allowed")
	site: str | None = Field(None, description="Website")
	weight_min: int | None = Field(None, description="Minimum weight limit")
	weight_max: int | None = Field(None, description="Maximum weight limit")
	fulfillment: bool | None = Field(None, description="Fulfillment available")


class DeliveryPointSearchResult(BaseModel):
	points: list[DeliveryPoint] = Field(..., description="Delivery points")
	total: int | None = Field(None, description="Total points count")



from pydantic import BaseModel, Field
from . import Location, Service, CDEKError, Money
from ..enums import OrderType, TariffCode, DeliveryMode, CountryCode


class OrderPackage(BaseModel):
	number: str = Field(..., description="Package number")
	weight: int = Field(..., description="Weight in grams", gt=0)
	length: int | None = Field(None, description="Length in centimeters")
	width: int | None = Field(None, description="Width in centimeters")
	height: int | None = Field(None, description="Height in centimeters")
	comment: str | None = Field(None, description="Package comment")
	items: list["OrderItem"] | None = Field(None, description="Items in package")


class OrderItem(BaseModel):
	name: str = Field(..., description="Item name")
	ware_key: str = Field(..., description="Item identifier")
	payment: Money = Field(..., description="Item cost")
	cost: float = Field(..., description="Declared value")
	weight: int = Field(..., description="Weight in grams")
	amount: int = Field(..., description="Quantity")
	name_i18n: str | None = Field(None, description="Item name in English")
	brand: str | None = Field(None, description="Brand")
	country_code: CountryCode | None = Field(None, description="Country of origin code")
	material: int | None = Field(None, description="Material code")
	wifi_gsm: bool | None = Field(None, description="Contains WiFi/GSM")
	url: str | None = Field(None, description="Product URL")


class OrderSender(BaseModel):
	company: str | None = Field(None, description="Company name")
	name: str = Field(..., description="Sender name")
	email: str | None = Field(None, description="Email")
	phones: list[str] = Field(..., description="Phone numbers")
	passport_series: str | None = Field(None, description="Passport series")
	passport_number: str | None = Field(None, description="Passport number")
	passport_date_of_issue: str | None = Field(None, description="Passport issue date")
	passport_organization: str | None = Field(
		None, description="Passport issuing organization"
	)
	tin: str | None = Field(None, description="Tax identification number")
	passport_date_of_birth: str | None = Field(None, description="Date of birth")


class OrderRecipient(BaseModel):
	company: str | None = Field(None, description="Company name")
	name: str = Field(..., description="Recipient name")
	email: str | None = Field(None, description="Email")
	phones: list[str] = Field(..., description="Phone numbers")
	passport_series: str | None = Field(None, description="Passport series")
	passport_number: str | None = Field(None, description="Passport number")
	passport_date_of_issue: str | None = Field(None, description="Passport issue date")
	passport_organization: str | None = Field(
		None, description="Passport issuing organization"
	)
	tin: str | None = Field(None, description="Tax identification number")
	passport_date_of_birth: str | None = Field(None, description="Date of birth")


class OrderRequest(BaseModel):
	type: OrderType = Field(
		..., description="Order type: 1 - online store, 2 - regular delivery"
	)
	number: str = Field(..., description="Order number")
	tariff_code: TariffCode = Field(..., description="Tariff code")
	comment: str | None = Field(None, description="Order comment")
	developer_key: str | None = Field(None, description="Developer key")
	shipment_point: str | None = Field(None, description="Shipment point code")
	delivery_point: str | None = Field(None, description="Delivery point code")
	date_invoice: str | None = Field(None, description="Invoice date")
	shipper_name: str | None = Field(None, description="Shipper name")
	shipper_address: str | None = Field(None, description="Shipper address")
	delivery_recipient_cost: Money | None = Field(
		None, description="Delivery cost paid by recipient"
	)
	delivery_recipient_cost_adv: list[Money] | None = Field(
		None, description="Additional delivery costs"
	)
	from_location: Location = Field(..., description="Origin location")
	to_location: Location = Field(..., description="Destination location")
	packages: list[OrderPackage] = Field(..., description="Packages", min_items=1)
	sender: OrderSender = Field(..., description="Sender information")
	recipient: OrderRecipient = Field(..., description="Recipient information")
	services: list[Service] | None = Field(None, description="Additional services")


class OrderResponse(BaseModel):
	entity: dict = Field(..., description="Created/updated order entity")
	requests: list[dict] | None = Field(None, description="Related requests")


class OrderInfo(BaseModel):
	uuid: str = Field(..., description="Order UUID")
	type: OrderType = Field(..., description="Order type")
	number: str = Field(..., description="Order number")
	tariff_code: TariffCode = Field(..., description="Tariff code")
	tariff_name: str = Field(..., description="Tariff name")
	tariff_description: str = Field(..., description="Tariff description")
	delivery_mode: DeliveryMode = Field(..., description="Delivery mode")
	delivery_detail: dict = Field(..., description="Delivery details")
	from_location: Location = Field(..., description="Origin location")
	to_location: Location = Field(..., description="Destination location")
	packages: list[OrderPackage] = Field(..., description="Packages")
	sender: OrderSender = Field(..., description="Sender information")
	recipient: OrderRecipient = Field(..., description="Recipient information")
	services: list[Service] | None = Field(None, description="Additional services")
	statuses: list[dict] | None = Field(None, description="Order statuses")
	calls: list[dict] | None = Field(None, description="Delivery calls")
	errors: list[CDEKError] | None = Field(None, description="Errors")


class OrderSearchParams(BaseModel):
	cdek_number: str | None = Field(None, description="CDEK order number")
	im_number: str | None = Field(None, description="IM number")
	order_uuid: str | None = Field(None, description="Order UUID")
	date_first: str | None = Field(None, description="Start date filter")
	date_last: str | None = Field(None, description="End date filter")
	size: int | None = Field(1000, description="Results per page")
	page: int | None = Field(0, description="Page number")



from pydantic import BaseModel, Field
from . import Location, Contact
from ..enums import WeightLimit, DimensionLimit


class CourierRequest(BaseModel):
	intake_date: str = Field(..., description="Pickup date (YYYY-MM-DD)")
	intake_time_from: str = Field(..., description="Pickup time from (HH:MM)")
	intake_time_to: str = Field(..., description="Pickup time to (HH:MM)")
	lunch_time_from: str | None = Field(None, description="Lunch break from (HH:MM)")
	lunch_time_to: str | None = Field(None, description="Lunch break to (HH:MM)")
	name: str = Field(..., description="Contact name")
	weight: int = Field(
		..., description="Total weight in grams", ge=WeightLimit.MIN_WEIGHT
	)
	length: int | None = Field(
		None, description="Length in centimeters", ge=DimensionLimit.MIN_DIMENSION
	)
	width: int | None = Field(
		None, description="Width in centimeters", ge=DimensionLimit.MIN_DIMENSION
	)
	height: int | None = Field(
		None, description="Height in centimeters", ge=DimensionLimit.MIN_DIMENSION
	)
	comment: str | None = Field(None, description="Comment")
	from_location: Location = Field(..., description="Pickup location")
	sender: Contact = Field(..., description="Sender contact information")
	need_call: bool | None = Field(None, description="Need to call before pickup")


class CourierResponse(BaseModel):
	entity: dict = Field(..., description="Created courier request entity")
	requests: list[dict] | None = Field(None, description="Related requests")


class CourierInfo(BaseModel):
	uuid: str = Field(..., description="Request UUID")
	intake_number: str | None = Field(None, description="Intake number")
	intake_date: str = Field(..., description="Pickup date")
	intake_time_from: str = Field(..., description="Pickup time from")
	intake_time_to: str = Field(..., description="Pickup time to")
	lunch_time_from: str | None = Field(None, description="Lunch break from")
	lunch_time_to: str | None = Field(None, description="Lunch break to")
	name: str = Field(..., description="Contact name")
	weight: int = Field(..., description="Total weight")
	length: int | None = Field(None, description="Length")
	width: int | None = Field(None, description="Width")
	height: int | None = Field(None, description="Height")
	comment: str | None = Field(None, description="Comment")
	from_location: Location = Field(..., description="Pickup location")
	sender: Contact = Field(..., description="Sender contact information")
	statuses: list[dict] | None = Field(None, description="Request statuses")


class CourierSearchParams(BaseModel):
	intake_number: str | None = Field(None, description="Intake number")
	date_first: str | None = Field(None, description="Start date filter")
	date_last: str | None = Field(None, description="End date filter")
	size: int | None = Field(1000, description="Results per page")
	page: int | None = Field(0, description="Page number")



from pydantic import BaseModel, Field
from ..enums import CountryCode, ServiceCode, Currency, VATRate


class Coordinates(BaseModel):
	latitude: float = Field(..., description="Latitude")
	longitude: float = Field(..., description="Longitude")


class Location(BaseModel):
	code: int | None = Field(None, description="City code from CDEK database")
	postal_code: str | None = Field(None, description="Postal code")
	country_code: CountryCode | None = Field(
		None, description="Country code in ISO_3166-1_alpha-2 format"
	)
	region: str | None = Field(None, description="Region name")
	region_code: int | None = Field(None, description="Region code")
	city: str | None = Field(None, description="City name")
	fias_guid: str | None = Field(None, description="FIAS code")
	kladr_code: str | None = Field(None, description="KLADR code")
	address: str | None = Field(None, description="Street address")


class Service(BaseModel):
	code: ServiceCode = Field(..., description="Service code")
	parameter: str | None = Field(None, description="Service parameter")


class CDEKError(BaseModel):
	code: str = Field(..., description="Error code")
	message: str = Field(..., description="Error message")


class Money(BaseModel):
	value: float = Field(..., description="Amount value")
	vat_sum: float | None = Field(None, description="VAT amount")
	vat_rate: VATRate | None = Field(None, description="VAT rate")
	currency: Currency | None = Field(None, description="Currency code")


class Contact(BaseModel):
	name: str = Field(..., description="Contact name")
	phones: list[str] = Field(..., description="Phone numbers")
	email: str | None = Field(None, description="Email address")
	passport_series: str | None = Field(None, description="Passport series")
	passport_number: str | None = Field(None, description="Passport number")
	passport_date_of_issue: str | None = Field(None, description="Passport issue date")
	passport_organization: str | None = Field(
		None, description="Passport issuing organization"
	)
	tin: str | None = Field(None, description="Tax identification number")
	passport_date_of_birth: str | None = Field(None, description="Date of birth")



import re
import sys
from contextlib import asynccontextmanager
from typing import Literal

import aiohttp.client_exceptions
from aiohttp import ClientSession
from loguru import logger


class APIClient:
    _base_url: str = None
    _client_name: str = __qualname__
    _debug_mode: bool = False

    @asynccontextmanager
    async def _get_cached_auth_data(self):
        yield

    async def _request(
        self,
        method: Literal["GET", "POST", "PUT", "PATCH", "DELETE", "OPTIONS"],
        path: str,
        params: dict = None,
        json: dict = None,
        **kwargs,
    ):
        async with (
            self._get_cached_auth_data() as auth,
            ClientSession(base_url=self._base_url) as client,
        ):
            assert path.startswith("/"), "invalid request path"
            assert re.match(r"^(https:|http:|\.)\S*", self._base_url), (
                f"invalid base_url for {self._client_name}"
            )

            headers = None

            if auth is not None:
                headers = {"Authorization": f"Bearer {auth.access_token}"}

            request = await client.request(
                method=method,
                url=path,
                headers=headers,
                params=params,
                json=json,
                **kwargs,
            )

            if not request.ok:
                if self._debug_mode:
                    logger.error(
                        f"[{request.status}] {method} URI: {self._base_url + path}, {params=}, {json=};\n{await request.text()}"
                    )
                raise aiohttp.client_exceptions.ClientError(
                    detail=f"Failed to perform {self._base_url + path} {method} request, {params=}, {json=}; {await request.text()}"
                )

            data = await request.json()

            if self._debug_mode:
                logger.debug(
                    f"[{request.status}] {method} URI: {self._base_url + path}, {params=}, {json=}"
                )

            return data

    async def _http_method(
        self, path: str, params: dict = None, json: dict = None, **kwargs
    ):
        return await self._request(
            method=sys._getframe().f_back.f_code.co_name.upper()[1:],
            path=path,
            params=params,
            json=json,
            **kwargs,
        )

    async def _get(self, path: str, params: dict = None, json: dict = None, **kwargs):
        return await self._http_method(path=path, params=params, json=json, **kwargs)

    async def _post(self, path: str, params: dict = None, json: dict = None, **kwargs):
        return await self._http_method(path=path, params=params, json=json, **kwargs)

    async def _put(self, path: str, params: dict = None, json: dict = None, **kwargs):
        return await self._http_method(path=path, params=params, json=json, **kwargs)

    async def _patch(self, path: str, params: dict = None, json: dict = None, **kwargs):
        return await self._http_method(path=path, params=params, json=json, **kwargs)

    async def _delete(self, path: str, params: dict = None, json: dict = None, **kwargs):
        return await self._http_method(path=path, params=params, json=json, **kwargs)



from typing import Any
from pydantic import ValidationError
from loguru import logger

from ..models import (
	WebhookRequest,
	WebhookResponse,
	WebhookInfo,
)

from ..enums import WebhookType


class WebhooksMixin:
	async def subscribe_webhook(
		self, url: str, webhook_type: WebhookType = WebhookType.ORDER_STATUS
	) -> WebhookResponse:
		try:
			webhook_request = WebhookRequest(url=url, type=webhook_type)
			response = await self._post(
				"/v2/webhooks", json=webhook_request.model_dump(exclude_none=True)
			)
			return WebhookResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in subscribe_webhook: {e}")
			raise

	async def get_webhooks(self) -> list[WebhookInfo]:
		try:
			response = await self._get("/v2/webhooks")
			if isinstance(response, list):
				return [WebhookInfo(**webhook) for webhook in response]
			return [WebhookInfo(**response)]
		except ValidationError as e:
			logger.error(f"Validation error in get_webhooks: {e}")
			raise

	async def delete_webhook(self, uuid: str) -> dict[str, Any]:
		try:
			response = await self._delete(f"/v2/webhooks/{uuid}")
			return response
		except ValidationError as e:
			logger.error(f"Validation error in delete_webhook: {e}")
			raise



import uuid

from pydantic import BaseModel, Field
from ..enums import CountryCode, Language


class RegionSearchParams(BaseModel):
	country_codes: list[CountryCode] | None = Field(None, description="Country codes")
	region_code: int | None = Field(None, description="Region code")
	size: int | None = Field(1000, description="Results per page")
	page: int | None = Field(0, description="Page number")
	lang: Language | None = Field(Language.RU, description="Language")


class CitySearchParams(BaseModel):
	country_codes: list[CountryCode] | None = Field(None, description="Country codes")
	region_code: int | None = Field(None, description="Region code")
	city_code: int | None = Field(None, description="City code")
	postal_code: str | None = Field(None, description="Postal code")
	city: str | None = Field(None, description="City name")
	fias_guid: str | None = Field(None, description="FIAS GUID")
	kladr_code: str | None = Field(None, description="KLADR code")
	size: int | None = Field(1000, description="Results per page")
	page: int | None = Field(0, description="Page number")
	lang: Language | None = Field(Language.RU, description="Language")


class Region(BaseModel):
	country_code: CountryCode = Field(..., description="Country code")
	country: str = Field(..., description="Country name")
	region: str = Field("", description="Region name")
	region_code: int = Field(..., description="Region code")


class City(BaseModel):
	country_code: CountryCode = Field(..., description="Country code")
	country: str | None = Field(None, description="Country name")
	region: str | None = Field(None, description="Region name")
	region_code: int | None = Field(None, description="Region code")
	city: str = Field(..., description="City name")
	city_code: int = Field(..., alias="code", description="City code")
	postal_code: str | None = Field(None, description="Postal code")
	fias_guid: str | None = Field(None, description="FIAS GUID")
	kladr_code: str | None = Field(None, description="KLADR code")
	longitude: float | None = Field(None, description="Longitude")
	latitude: float | None = Field(None, description="Latitude")
	time_zone: str | None = Field(None, description="Time zone")
	payment_limit: float | None = Field(None, description="Payment limit")

class CityFromSearch(BaseModel):
	city_uuid: uuid.UUID = Field(..., description="City UUID")
	code: int = Field(..., description="City code")
	full_name: str = Field(..., description="Full name of the city")



__version__ = "0.1.6"
__author__ = "Alexey P."
__email__ = "i@apechenin.ru"

from .client.cdek import CDEKAPIClient
from .client.common import APIClient

__all__ = [
    "CDEKAPIClient",
    "APIClient",
]




from pydantic import BaseModel, Field
from ..enums import WebhookType


class WebhookRequest(BaseModel):
	url: str = Field(..., description="Webhook URL")
	type: WebhookType = Field(WebhookType.ORDER_STATUS, description="Webhook type")


class WebhookResponse(BaseModel):
	uuid: str = Field(..., description="Webhook UUID")
	url: str = Field(..., description="Webhook URL")
	type: WebhookType = Field(..., description="Webhook type")
	account: str | None = Field(None, description="Account identifier")


class WebhookInfo(BaseModel):
	uuid: str = Field(..., description="Webhook UUID")
	url: str = Field(..., description="Webhook URL")
	type: WebhookType = Field(..., description="Webhook type")
	account: str | None = Field(None, description="Account identifier")
	created_datetime: str | None = Field(None, description="Creation datetime")
	is_active: bool | None = Field(None, description="Is webhook active")



from pydantic import ValidationError
from loguru import logger
from ..models import (
	TariffRequest,
	TariffResponse,
	TariffListRequest,
	CustomsDutyRequest,
	CustomsDutyResponse,
)


class CalculatorMixin:
	async def calculate_tariff(cls, request: TariffRequest) -> TariffResponse:
		try:
			response = await cls._post(
				"/v2/calculator/tariff", json=request.model_dump(exclude_none=True)
			)
			return TariffResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in calculate_tariff: {e}")
			raise

	async def calculate_tariff_list(
		cls, request: TariffListRequest
	) -> list[TariffResponse]:
		try:
			response = await cls._post(
				"/v2/calculator/tarifflist", json=request.model_dump(exclude_none=True)
			)
			tariff_codes = response.get("tariff_codes", [])
			return [TariffResponse(**tariff) for tariff in tariff_codes]
		except ValidationError as e:
			logger.error(f"Validation error in calculate_tariff_list: {e}")
			raise

	async def calculate_customs_duty(
		cls, request: CustomsDutyRequest
	) -> CustomsDutyResponse:
		try:
			response = await cls._post(
				"/v2/calculator/customs-duty",
				json=request.model_dump(exclude_none=True),
			)
			return CustomsDutyResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in calculate_customs_duty: {e}")
			raise



from pydantic import BaseModel, Field
from . import Location, Service, CDEKError
from ..enums import TariffCode, Currency, Language, DeliveryMode


class CalculatorPackage(BaseModel):
	weight: int = Field(..., description="Total weight in grams", gt=0)
	length: int | None = Field(None, description="Length in centimeters", gt=0)
	width: int | None = Field(None, description="Width in centimeters", gt=0)
	height: int | None = Field(None, description="Height in centimeters", gt=0)
	volume_weight: int | None = Field(
		None, description="Volume weight (auto-calculated if not specified)"
	)


class TariffRequest(BaseModel):
	tariff_code: TariffCode | None = Field(None, description="Tariff code")
	from_location: Location = Field(..., description="Origin location")
	to_location: Location = Field(..., description="Destination location")
	packages: list[CalculatorPackage] = Field(
		..., description="Packages list", min_items=1
	)
	services: list[Service] | None = Field(None, description="Additional services")
	date: str | None = Field(None, description="Planned shipment date (YYYY-MM-DD)")
	currency: Currency | None = Field(
		Currency.RUB, description="Currency for calculation"
	)
	lang: Language | None = Field(
		Language.RU, description="Language for tariff information output"
	)


class TariffServiceResponse(BaseModel):
	code: str = Field(..., description="Service code")
	name: str = Field(..., description="Service name")
	sum: float = Field(..., description="Service cost")


class TariffResponse(BaseModel):
	tariff_code: TariffCode = Field(..., description="Tariff code")
	tariff_name: str = Field(..., description="Tariff name")
	tariff_description: str = Field(..., description="Tariff description")
	delivery_mode: DeliveryMode = Field(..., description="Tariff mode")
	delivery_sum: float = Field(..., description="Delivery cost")
	period_min: int = Field(..., description="Minimum delivery time in days")
	period_max: int = Field(..., description="Maximum delivery time in days")
	weight_calc: int = Field(..., description="Calculated weight in grams")
	total_sum: float = Field(..., description="Total cost")
	currency: Currency = Field(..., description="Currency")
	services: list[TariffServiceResponse] | None = Field(None, description="Services")
	errors: list[CDEKError] | None = Field(None, description="Errors")


class TariffListRequest(BaseModel):
	from_location: Location = Field(..., description="Origin location")
	to_location: Location = Field(..., description="Destination location")
	packages: list[CalculatorPackage] = Field(
		..., description="Packages list", min_items=1
	)
	services: list[Service] | None = Field(None, description="Additional services")
	date: str | None = Field(None, description="Planned shipment date (YYYY-MM-DD)")
	currency: Currency | None = Field(
		Currency.RUB, description="Currency for calculation"
	)
	lang: Language | None = Field(
		Language.RU, description="Language for tariff information output"
	)


class CustomsDutyRequest(BaseModel):
	from_location: Location = Field(..., description="Origin location")
	to_location: Location = Field(..., description="Destination location")
	packages: list[CalculatorPackage] = Field(
		..., description="Packages list", min_items=1
	)
	currency: Currency | None = Field(
		Currency.RUB, description="Currency for calculation"
	)


class CustomsDutyResponse(BaseModel):
	duty_sum: float = Field(..., description="Customs duty amount")
	vat_sum: float = Field(..., description="VAT amount")
	total_sum: float = Field(..., description="Total customs payments")
	currency: Currency = Field(..., description="Currency")
	errors: list[CDEKError] | None = Field(None, description="Errors")



from .base import Coordinates, Location, Service, CDEKError, Money, Contact
from .calculator import (
	CalculatorPackage,
	TariffRequest,
	TariffServiceResponse,
	TariffResponse,
	TariffListRequest,
	CustomsDutyRequest,
	CustomsDutyResponse,
)
from .orders import (
	OrderPackage,
	OrderItem,
	OrderSender,
	OrderRecipient,
	OrderRequest,
	OrderResponse,
	OrderInfo,
	OrderSearchParams,
)
from .locations import (
	RegionSearchParams,
	CitySearchParams,
	Region,
	City,
)
from .delivery_points import (
	DeliveryPointFilter,
	WorkTime,
	DeliveryPoint,
	DeliveryPointSearchResult,
)
from .courier import (
	CourierRequest,
	CourierResponse,
	CourierInfo,
	CourierSearchParams,
)
from .printing import (
	PrintRequest,
	PrintResponse,
)
from .webhooks import (
	WebhookRequest,
	WebhookResponse,
	WebhookInfo,
)

__all__ = [
	"Coordinates",
	"Location",
	"Service",
	"CDEKError",
	"Money",
	"Contact",
	"CalculatorPackage",
	"TariffRequest",
	"TariffServiceResponse",
	"TariffResponse",
	"TariffListRequest",
	"CustomsDutyRequest",
	"CustomsDutyResponse",
	"OrderPackage",
	"OrderItem",
	"OrderSender",
	"OrderRecipient",
	"OrderRequest",
	"OrderResponse",
	"OrderInfo",
	"OrderSearchParams",
	"RegionSearchParams",
	"CitySearchParams",
	"Region",
	"City",
	"DeliveryPointFilter",
	"WorkTime",
	"DeliveryPoint",
	"DeliveryPointSearchResult",
	"CourierRequest",
	"CourierResponse",
	"CourierInfo",
	"CourierSearchParams",
	"PrintRequest",
	"PrintResponse",
	"WebhookRequest",
	"WebhookResponse",
	"WebhookInfo",
]



from .enums import (
	OrderType,
	TariffCode,
	DeliveryMode,
	PackageType,
	ServiceCode,
	OrderStatus,
	PackageStatus,
	DeliveryPointType,
	PaymentMethod,
	WeightLimit,
	DimensionLimit,
	CountryCode,
	Currency,
	VATRate,
	PrintFormat,
	WebhookType,
	Language,
)


__all__ = [
	"OrderType",
	"TariffCode",
	"DeliveryMode",
	"PackageType",
	"ServiceCode",
	"OrderStatus",
	"PackageStatus",
	"DeliveryPointType",
	"PaymentMethod",
	"WeightLimit",
	"DimensionLimit",
	"CountryCode",
	"Currency",
	"VATRate",
	"PrintFormat",
	"WebhookType",
	"Language",
]



from pydantic import ValidationError
from loguru import logger
from ..models import (
	CourierRequest,
	CourierResponse,
	CourierInfo,
	CourierSearchParams,
)


class CourierMixin:
	async def create_courier_request(self, request: CourierRequest) -> CourierResponse:
		try:
			response = await self._post(
				"/v2/intakes", json=request.model_dump(exclude_none=True)
			)
			return CourierResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in create_courier_request: {e}")
			raise

	async def get_courier_request(self, uuid: str) -> CourierInfo:
		try:
			response = await self._get(f"/v2/intakes/{uuid}")
			return CourierInfo(**response)
		except ValidationError as e:
			logger.error(f"Validation error in get_courier_request: {e}")
			raise

	async def get_courier_requests(
		self, params: CourierSearchParams | None = None
	) -> list[CourierInfo]:
		try:
			search_params = params.model_dump(exclude_none=True) if params else {}
			response = await self._get("/v2/intakes", params=search_params)
			if isinstance(response, list):
				return [CourierInfo(**request) for request in response]
			return [CourierInfo(**response)]
		except ValidationError as e:
			logger.error(f"Validation error in get_courier_requests: {e}")
			raise

	async def delete_courier_request(self, uuid: str) -> CourierResponse:
		try:
			response = await self._delete(f"/v2/intakes/{uuid}")
			return CourierResponse(**response)
		except ValidationError as e:
			logger.error(f"Validation error in delete_courier_request: {e}")
			raise



from pydantic import ValidationError
from loguru import logger

from ..enums import CountryCode
from ..models import (
	RegionSearchParams,
	CitySearchParams,
	Region,
	City,
)
from ..models.locations import CityFromSearch


class LocationsMixin:
    async def get_regions(
        self, params: RegionSearchParams | None = None
    ) -> list[Region]:
        try:
            search_params = (
                params.model_dump(exclude_none=True)
                if params
                else {"size": 1000, "page": 0}
            )
            response = await self._get("/v2/location/regions", params=search_params)
            if isinstance(response, list):
                return [Region(**region) for region in response]
            return [Region(**response)]
        except ValidationError as e:
            logger.error(f"Validation error in get_regions: {e}")
            raise

    async def get_cities(self, params: CitySearchParams | None = None) -> list[City]:
        try:
            search_params = (
                params.model_dump(exclude_none=True)
                if params
                else {"size": 1000, "page": 0}
            )
            response = await self._get("/v2/location/cities", params=search_params)
            if isinstance(response, list):
                return [City(**city) for city in response]
            return [City(**response)]
        except ValidationError as e:
            logger.error(f"Validation error in get_cities: {e}")
            raise

    async def get_approximate_city(self, query: str, country_code: CountryCode = CountryCode.RU) -> list[CityFromSearch]:
        try:

            response = await self._get(
                "/v2/location/suggest/cities", params={
                    "name": query,
                    "country_code": country_code
                }
            )
            if isinstance(response, list):
                return [CityFromSearch(**city) for city in response]
            return [CityFromSearch(**response)]
        except ValidationError as e:
            logger.error(f"Validation error in get_approximate_cities: {e}")
            raise
